@@ -2,7 +2,7 @@
  *  SubCircuit -- An implementation of the Ullmann Subgraph Isomorphism
  *                algorithm for coarse grain logic networks
  *
- *  Copyright (C) 2013  Clifford Wolf <clifford@clifford.at>
+ *  Copyright (C) 2013  Claire Xenia Wolf <claire@yosyshq.com>
  *
  *  Permission to use, copy, modify, and/or distribute this software for any
  *  purpose with or without fee is hereby granted, provided that the above
@@ -316,7 +316,7 @@ class SubCircuit::SolverWorker
 
 	// helper functions for handling permutations
 
-	static const int maxPermutationsLimit = 1000000;
+	static constexpr int maxPermutationsLimit = 1000000;
 
 	static int numberOfPermutations(const std::vector<std::string> &list)
 	{
@@ -1689,4 +1689,3 @@ void SubCircuit::Solver::clearConfig()
 {
 	worker->clearConfig();
 }
-
