@@ -631,8 +631,8 @@ struct MemoryDffPass : public Pass {
 		log("\n");
 		log("    memory_dff [-no-rw-check] [selection]\n");
 		log("\n");
-		log("This pass detects DFFs at memory read ports and merges them into the memory port.\n");
-		log("I.e. it consumes an asynchronous memory port and the flip-flops at its\n");
+		log("This pass detects DFFs at memory read ports and merges them into the memory\n");
+		log("port. I.e. it consumes an asynchronous memory port and the flip-flops at its\n");
 		log("interface and yields a synchronous memory port.\n");
 		log("\n");
 		log("    -no-rw-check\n");
