@@ -271,6 +271,24 @@ void abc9_module(RTLIL::Design *design, std::string script_file, std::string exe
 	abc9_output_filter filt(tempdir_name, show_tempdir);
 	int ret = run_command(buffer, std::bind(&abc9_output_filter::next_line, filt, std::placeholders::_1));
 #else
+	string temp_stdouterr_name = stringf("%s/stdouterr.txt", tempdir_name.c_str());
+	FILE *temp_stdouterr_w = fopen(temp_stdouterr_name.c_str(), "w");
+	if (temp_stdouterr_w == NULL)
+		log_error("ABC: cannot open a temporary file for output redirection");
+	fflush(stdout);
+	fflush(stderr);
+	FILE *old_stdout = fopen(temp_stdouterr_name.c_str(), "r"); // need any fd for renumbering
+	FILE *old_stderr = fopen(temp_stdouterr_name.c_str(), "r"); // need any fd for renumbering
+#if defined(__wasm)
+#define fd_renumber(from, to) (void)__wasi_fd_renumber(from, to)
+#else
+#define fd_renumber(from, to) dup2(from, to)
+#endif
+	fd_renumber(fileno(stdout), fileno(old_stdout));
+	fd_renumber(fileno(stderr), fileno(old_stderr));
+	fd_renumber(fileno(temp_stdouterr_w), fileno(stdout));
+	fd_renumber(fileno(temp_stdouterr_w), fileno(stderr));
+	fclose(temp_stdouterr_w);
 	// These needs to be mutable, supposedly due to getopt
 	char *abc9_argv[5];
 	string tmp_script_name = stringf("%s/abc.script", tempdir_name.c_str());
@@ -284,6 +302,17 @@ void abc9_module(RTLIL::Design *design, std::string script_file, std::string exe
 	free(abc9_argv[1]);
 	free(abc9_argv[2]);
 	free(abc9_argv[3]);
+	fflush(stdout);
+	fflush(stderr);
+	fd_renumber(fileno(old_stdout), fileno(stdout));
+	fd_renumber(fileno(old_stderr), fileno(stderr));
+	fclose(old_stdout);
+	fclose(old_stderr);
+	std::ifstream temp_stdouterr_r(temp_stdouterr_name);
+	abc9_output_filter filt(tempdir_name, show_tempdir);
+	for (std::string line; std::getline(temp_stdouterr_r, line); )
+		filt.next_line(line + "\n");
+	temp_stdouterr_r.close();
 #endif
 	if (ret != 0) {
 		if (check_file_exists(stringf("%s/output.aig", tempdir_name.c_str())))
